from conan.internal.model.cpp_info import CppInfo


class TestCppInfoComponents:

    def test_components_set(self):
        cpp_info = CppInfo(set_defaults=True)
        cpp_info.components["liba"].libs = ["liba"]
        cpp_info.components["libb"].includedirs.append("includewhat")
        cpp_info.components["libc"].libs.append("thelibc")
        assert list(cpp_info.components.keys()) == ["liba", "libb", "libc"]

        assert cpp_info.components["libb"].includedirs == ["include", "includewhat"]
        assert cpp_info.components["libc"].libs == ["thelibc"]

        import pytest
        with pytest.raises(AttributeError):
            cpp_info.required_components = ["liba::liba"]

    def test_no_components_inside_components(self):
        cpp_info = CppInfo()
        import pytest
        with pytest.raises(AttributeError):
            _ = cpp_info.components["libb"].components

    def test_cppinfo_components_dirs(self):
        info = CppInfo()
        info.components["OpenSSL"].includedirs = ["include"]
        info.components["OpenSSL"].libdirs = ["lib"]
        info.components["OpenSSL"].builddirs = ["build"]
        info.components["OpenSSL"].bindirs = ["bin"]
        info.components["OpenSSL"].resdirs = ["res"]
        info.components["Crypto"].includedirs = ["headers"]
        info.components["Crypto"].libdirs = ["libraries"]
        info.components["Crypto"].builddirs = ["build_scripts"]
        info.components["Crypto"].bindirs = ["binaries"]
        info.components["Crypto"].resdirs = ["resources"]
        assert ["include"] == info.components["OpenSSL"].includedirs
        assert ["lib"] == info.components["OpenSSL"].libdirs
        assert ["build"] == info.components["OpenSSL"].builddirs
        assert ["bin"] == info.components["OpenSSL"].bindirs
        assert ["res"] == info.components["OpenSSL"].resdirs
        assert ["headers"] == info.components["Crypto"].includedirs
        assert ["libraries"] == info.components["Crypto"].libdirs
        assert ["build_scripts"] == info.components["Crypto"].builddirs
        assert ["binaries"] == info.components["Crypto"].bindirs
        assert ["resources"] == info.components["Crypto"].resdirs

        info.components["Crypto"].includedirs = ["different_include"]
        info.components["Crypto"].libdirs = ["different_lib"]
        info.components["Crypto"].builddirs = ["different_build"]
        info.components["Crypto"].bindirs = ["different_bin"]
        info.components["Crypto"].resdirs = ["different_res"]
        assert ["different_include"] == info.components["Crypto"].includedirs
        assert ["different_lib"] == info.components["Crypto"].libdirs
        assert ["different_build"] == info.components["Crypto"].builddirs
        assert ["different_bin"] == info.components["Crypto"].bindirs
        assert ["different_res"] == info.components["Crypto"].resdirs

        info.components["Crypto"].includedirs.extend(["another_include"])
        info.components["Crypto"].includedirs.append("another_other_include")
        info.components["Crypto"].libdirs.extend(["another_lib"])
        info.components["Crypto"].libdirs.append("another_other_lib")
        info.components["Crypto"].builddirs.extend(["another_build"])
        info.components["Crypto"].builddirs.append("another_other_build")
        info.components["Crypto"].bindirs.extend(["another_bin"])
        info.components["Crypto"].bindirs.append("another_other_bin")
        info.components["Crypto"].resdirs.extend(["another_res"])
        info.components["Crypto"].resdirs.append("another_other_res")
        assert ["different_include", "another_include", "another_other_include"] == \
                         info.components["Crypto"].includedirs
        assert ["different_lib", "another_lib", "another_other_lib"] == \
                         info.components["Crypto"].libdirs
        assert ["different_build", "another_build", "another_other_build"] == \
                         info.components["Crypto"].builddirs
        assert ["different_bin", "another_bin", "another_other_bin"] == \
                         info.components["Crypto"].bindirs
        assert ["different_res", "another_res", "another_other_res"] == \
                         info.components["Crypto"].resdirs



from conan.errors import ConanException


def is_multi_configuration(generator):
    if not generator:
        return False
    return "Visual" in generator or "Xcode" in generator or "Multi-Config" in generator


def parse_extra_variable(source, key, value):
    CMAKE_CACHE_TYPES = ["BOOL", "FILEPATH", "PATH", "STRING", "INTERNAL"]
    if isinstance(value, str):
        return f"\"{value}\""
    elif isinstance(value, (int, float)):
        return value
    elif isinstance(value, dict):
        var_value = parse_extra_variable(source, key, value.get("value"))
        is_force = value.get("force")
        if is_force:
            if not isinstance(is_force, bool):
                raise ConanException(f'{source} "{key}" "force" must be a boolean')
        is_cache = value.get("cache")
        if is_cache:
            if not isinstance(is_cache, bool):
                raise ConanException(f'{source} "{key}" "cache" must be a boolean')
            var_type = value.get("type")
            if not var_type:
                raise ConanException(f'{source} "{key}" needs "type" defined for cache variable')
            if var_type not in CMAKE_CACHE_TYPES:
                raise ConanException(f'{source} "{key}" invalid type "{var_type}" for cache variable.'
                                     f' Possible types: {", ".join(CMAKE_CACHE_TYPES)}')
            # Set docstring as variable name if not defined
            docstring = value.get("docstring") or key
            force_str = " FORCE" if is_force else ""  # Support python < 3.11
            return f"{var_value} CACHE {var_type} \"{docstring}\"{force_str}"
        else:
            if is_force:
                raise ConanException(f'{source} "{key}" "force" is only allowed for cache variables')
            return var_value
    raise ConanException(f'{source} "{key}" has invalid type. Allowed types: str, int, float, dict,'
                         f' got {type(value)}')


def cmake_escape_value(v):
    return v.replace('\\', '\\\\').replace('$', '\\$').replace('"', '\\"')



import textwrap

from conan.test.utils.tools import TestClient, GenConanfile


class TestDeprecated:
    def test_no_deprecated(self):
        t = TestClient()
        t.save({'taskflow.py': GenConanfile("cpp-taskflow", "1.0").with_deprecated("''")})
        t.run("create taskflow.py")

        assert "Deprecated" not in t.out

    def test_deprecated_simple(self):
        t = TestClient()
        t.save({'taskflow.py': GenConanfile("cpp-taskflow", "1.0").with_deprecated("True")})
        t.run("create taskflow.py")

        assert "Deprecated\n    cpp-taskflow/1.0" in t.out
        assert "WARN: risk: There are deprecated packages in the graph" in t.out

        t.run("create taskflow.py --user=conan --channel=stable")
        assert "Deprecated\n    cpp-taskflow/1.0" in t.out

    def test_deprecated_with(self):
        t = TestClient()
        t.save({'taskflow.py': GenConanfile("cpp-taskflow", "1.0").with_deprecated('"taskflow"')})
        t.run("create taskflow.py")

        assert "Deprecated\n    cpp-taskflow/1.0: taskflow" in t.out

        t.run("create taskflow.py --user=conan --channel=stable")
        assert "Deprecated\n    cpp-taskflow/1.0@conan/stable: taskflow" in t.out

    def test_deprecated_custom_text(self):
        tc = TestClient()
        tc.save({"old/conanfile.py": GenConanfile("maths", 1.0).with_deprecated('"This is not secure, use maths/[>=2.0]"'),
                 "new/conanfile.py": GenConanfile("maths", 2.0)})

        tc.run("create new/conanfile.py")
        tc.run("create old/conanfile.py")
        assert "maths/1.0: This is not secure, use maths/[>=2.0]" in tc.out
        tc.run("install --requires=maths/1.0")
        assert "maths/1.0: This is not secure, use maths/[>=2.0]" in tc.out

    def test_deprecated_configure(self):
        tc = TestClient()
        conanfile = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.scm import Version

        class Pkg(ConanFile):
            name = "pkg"

            def configure(self):
                if Version(self.version) < "1.0":
                    self.deprecated = True
        """)
        tc.save({"conanfile.py": conanfile})

        tc.run("graph info . --version=0.0")
        assert "deprecated: True" in tc.out

        tc.run("graph info . --version=2.0")
        assert "deprecated: None" in tc.out



import json
import os
import textwrap

import pytest

from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.tools import TestClient


class TestToolRequires:
    def test_system_tool_require(self):
        client = TestClient(light=True)
        client.save({"conanfile.py": GenConanfile("pkg", "1.0").with_tool_requires("tool/1.0"),
                     "profile": "[platform_tool_requires]\ntool/1.0"})
        client.run("create . -pr=profile")
        assert "tool/1.0 - Platform" in client.out

    def test_system_tool_require_non_matching(self):
        """ if what is specified in [system_tool_require] doesn't match what the recipe requires, then
        the system_tool_require will not be used, and the recipe will use its declared version
        """
        client = TestClient(light=True)
        client.save({"tool/conanfile.py": GenConanfile("tool", "1.0"),
                     "conanfile.py": GenConanfile("pkg", "1.0").with_tool_requires("tool/1.0"),
                     "profile": "[system_tools]\ntool/1.1"})
        client.run("create tool")
        client.run("create . -pr=profile")
        assert "WARN: Profile [system_tools] is deprecated" in client.out
        assert "tool/1.0#60ed6e65eae112df86da7f6d790887fd - Cache" in client.out

    def test_system_tool_require_range(self):
        client = TestClient(light=True)
        client.save({"conanfile.py": GenConanfile("pkg", "1.0").with_tool_requires("tool/[>=1.0]"),
                     "profile": "[platform_tool_requires]\ntool/1.1"})
        client.run("create . -pr=profile")
        assert "tool/1.1 - Platform" in client.out

    def test_system_tool_require_range_non_matching(self):
        """ if what is specified in [system_tool_require] doesn't match what the recipe requires, then
        the system_tool_require will not be used, and the recipe will use its declared version
        """
        client = TestClient(light=True)
        client.save({"tool/conanfile.py": GenConanfile("tool", "1.1"),
                     "conanfile.py": GenConanfile("pkg", "1.0").with_tool_requires("tool/[>=1.0]"),
                     "profile": "[platform_tool_requires]\ntool/0.1"})
        client.run("create tool")
        client.run("create . -pr=profile")
        assert "tool/1.1#888bda2348dd2ddcf5960d0af63b08f7 - Cache" in client.out

    def test_system_tool_require_no_host(self):
        """
        system_tools must not affect host context
        """
        client = TestClient(light=True)
        client.save({"conanfile.py": GenConanfile("pkg", "1.0").with_requires("tool/1.0"),
                     "profile": "[platform_tool_requires]\ntool/1.0"})
        client.run("create . -pr=profile", assert_error=True)
        assert "ERROR: Package 'tool/1.0' not resolved: No remote defined" in client.out

    def test_graph_info_system_tool_require_range(self):
        """
        graph info doesn't crash
        """
        client = TestClient(light=True)
        client.save({"conanfile.py": GenConanfile("pkg", "1.0").with_tool_requires("tool/[>=1.0]"),
                     "profile": "[platform_tool_requires]\ntool/1.1"})
        client.run("graph info . -pr=profile")
        assert "tool/1.1 - Platform" in client.out

    def test_consumer_resolved_version(self):
        client = TestClient(light=True)
        conanfile = textwrap.dedent("""
            from conan import ConanFile
            class Pkg(ConanFile):
                tool_requires = "tool/[>=1.0]"

                def generate(self):
                    for r, _ in self.dependencies.items():
                        self.output.info(f"DEPENDENCY {r.ref}")
                """)
        client.save({"conanfile.py": conanfile,
                     "profile": "[platform_tool_requires]\ntool/1.1"})
        client.run("install . -pr=profile")
        assert "tool/1.1 - Platform" in client.out
        assert "conanfile.py: DEPENDENCY tool/1.1" in client.out

    def test_consumer_resolved_revision(self):
        client = TestClient(light=True)
        conanfile = textwrap.dedent("""
            from conan import ConanFile
            class Pkg(ConanFile):
                tool_requires = "tool/1.1"

                def generate(self):
                    for r, _ in self.dependencies.items():
                        self.output.info(f"DEPENDENCY {repr(r.ref)}")
                """)
        client.save({"conanfile.py": conanfile,
                     "profile": "[platform_tool_requires]\ntool/1.1#rev1"})
        client.run("install . -pr=profile")
        assert "tool/1.1 - Platform" in client.out
        assert "conanfile.py: DEPENDENCY tool/1.1#rev1" in client.out

        conanfile = textwrap.dedent("""
            from conan import ConanFile
            class Pkg(ConanFile):
               tool_requires = "tool/1.1#rev1"

               def generate(self):
                   for r, _ in self.dependencies.items():
                       self.output.info(f"DEPENDENCY {repr(r.ref)}")
               """)
        client.save({"conanfile.py": conanfile})
        client.run("install . -pr=profile")
        assert "tool/1.1 - Platform" in client.out
        assert "conanfile.py: DEPENDENCY tool/1.1#rev1" in client.out

    def test_consumer_unresolved_revision(self):
        """ if a recipe specifies an exact revision and so does the profiñe
        and it doesn't match, it is an error
        """
        client = TestClient(light=True)
        conanfile = textwrap.dedent("""
            from conan import ConanFile
            class Pkg(ConanFile):
                tool_requires = "tool/1.1#rev2"

                def generate(self):
                    for r, _ in self.dependencies.items():
                        self.output.info(f"DEPENDENCY {repr(r.ref)}")
                """)
        client.save({"conanfile.py": conanfile,
                     "profile": "[platform_tool_requires]\ntool/1.1#rev1"})
        client.run("install . -pr=profile", assert_error=True)
        assert "ERROR: Package 'tool/1.1' not resolved" in client.out

    def test_require_build_context(self):
        """ https://github.com/conan-io/conan/issues/15659
        app -> libcurl --(tool-require)--> libtool -> automake -> autoconf
                                             \\--(br)-> automake -> autoconf
        """
        c = TestClient(light=True)
        c.save({"autoconf/conanfile.py": GenConanfile("autoconf", "0.1"),
                "automake/conanfile.py": GenConanfile("automake", "0.1").with_requires("autoconf/0.1"),
                "libtool/conanfile.py": GenConanfile("libtool", "0.1").with_requires("automake/0.1")
                                                                      .with_tool_requires("automake/0.1"),
                "libcurl/conanfile.py": GenConanfile("libcurl", "0.1").with_tool_requires("libtool/0.1"),
                "app/conanfile.py": GenConanfile().with_requires("libcurl/0.1"),
                "profile_build": "[settings]\nos=Linux\n[platform_tool_requires]\nautomake/0.1"})
        c.run("create autoconf")
        c.run("create automake")
        c.run("export libtool")
        c.run("export libcurl")
        c.run("install app -pr:b=profile_build --build=missing")
        assert "Install finished successfully" in c.out


class TestToolRequiresLock:

    def test_system_tool_require_range(self):
        c = TestClient(light=True)
        c.save({"conanfile.py": GenConanfile("pkg", "1.0").with_tool_requires("tool/[>=1.0]"),
                "profile": "[platform_tool_requires]\ntool/1.1"})
        c.run("lock create . -pr=profile")
        assert "tool/1.1 - Platform" in c.out
        lock = json.loads(c.load("conan.lock"))
        assert lock["build_requires"] == ["tool/1.1"]

        c.run("install .", assert_error=True)
        assert "Package 'tool/1.1' not resolved: No remote defined" in c.out
        c.run("install . -pr=profile")
        assert "tool/1.1 - Platform" in c.out

        # even if we create a version within the range, it will error if not matching the profile
        c.save({"tool/conanfile.py": GenConanfile("tool", "1.1")})
        c.run("create tool")
        # if the profile points to another version it is an error, not in the lockfile
        c.save({"profile": "[platform_tool_requires]\ntool/1.2"})
        c.run("install . -pr=profile", assert_error=True)
        assert "ERROR: Requirement 'tool/1.2' not in lockfile" in c.out

        # if we relax the lockfile, we can still resolve to the platform_tool_requires
        # specified by the profile
        c.run("install . -pr=profile --lockfile-partial")
        assert "tool/1.2 - Platform" in c.out


class TestGenerators:
    def test_system_tool_require_range(self):
        client = TestClient()
        conanfile = textwrap.dedent("""
            from conan import ConanFile
            from conan.tools.cmake import CMakeDeps
            from conan.tools.gnu import PkgConfigDeps

            class Pkg(ConanFile):
                settings = "build_type"
                tool_requires = "tool/[>=1.0]"
                def generate(self):
                    deps = CMakeDeps(self)
                    deps.build_context_activated = ["tool"]
                    deps.generate()
                    deps = PkgConfigDeps(self)
                    deps.build_context_activated = ["tool"]
                    deps.generate()
            """)
        client.save({"conanfile.py": conanfile,
                     "profile": "[platform_tool_requires]\ntool/1.1"})
        client.run("install . -pr=profile")
        assert "tool/1.1 - Platform" in client.out
        assert not os.path.exists(os.path.join(client.current_folder, "tool-config.cmake"))
        assert not os.path.exists(os.path.join(client.current_folder, "tool.pc"))


class TestPackageID:
    """ if a consumer depends on recipe revision or package_id what happens
    """

    @pytest.mark.parametrize("package_id_mode", ["recipe_revision_mode", "full_package_mode"])
    def test_package_id_modes(self, package_id_mode):
        """ this test validates that the computation of the downstream consumers package_id
        doesnt break even if it depends on fields not existing in upstream system_tool_require, like revision
        or package_id
        """
        client = TestClient(light=True)
        client.save_home({"global.conf": f"core.package_id:default_build_mode={package_id_mode}"})
        client.save({"conanfile.py": GenConanfile("pkg", "1.0").with_tool_requires("dep/1.0"),
                     "profile": "[platform_tool_requires]\ndep/1.0"})
        client.run("create . -pr=profile")
        assert "dep/1.0 - Platform" in client.out

    def test_package_id_explicit_revision(self):
        """
        Changing the system_tool_require revision affects consumers if package_revision_mode=recipe_revision
        """
        client = TestClient(light=True)
        client.save_home({"global.conf": "core.package_id:default_build_mode=recipe_revision_mode"})
        client.save({"conanfile.py": GenConanfile("pkg", "1.0").with_tool_requires("dep/1.0"),
                     "profile": "[platform_tool_requires]\ndep/1.0#r1",
                     "profile2": "[platform_tool_requires]\ndep/1.0#r2"})
        client.run("create . -pr=profile")
        assert "dep/1.0#r1 - Platform" in client.out
        assert "pkg/1.0#27a56f09310cf1237629bae4104fe5bd:" \
               "ea0e320d94b4b70fcb3efbabf9ab871542f8f696 - Build" in client.out

        client.run("create . -pr=profile2")
        # pkg gets a new package_id because it is a different revision
        assert "dep/1.0#r2 - Platform" in client.out
        assert "pkg/1.0#27a56f09310cf1237629bae4104fe5bd:" \
               "334882884da082740e5a002a0b6fdb509a280159 - Build" in client.out



import pytest

from conan.internal.model.info import _VersionRepr
from conan.internal.model.version import Version


class TestVersionRepr:

    def test_text(self):
        v1 = Version("master+build2")
        vr = _VersionRepr(v1)
        assert vr.major() == "master"
        assert vr.minor() == "master"
        assert vr.patch() == "master"
        assert vr.pre() == "master"
        assert vr.build == "build2"
        assert vr.stable() == "master"

    def test_patch(self):
        v1 = Version("1.2.3-alpha1+build2")
        vr = _VersionRepr(v1)
        assert vr.major() == "1.Y.Z"
        assert vr.minor() == "1.2.Z"
        assert vr.patch() == "1.2.3"
        assert vr.pre() == "1.2.3-alpha1"
        assert vr.build == "build2"
        assert vr.stable() == "1.Y.Z"

        v1 = Version("1.2.3+build2")
        vr = _VersionRepr(v1)
        assert vr.major() == "1.Y.Z"
        assert vr.minor() == "1.2.Z"
        assert vr.patch() == "1.2.3"
        assert vr.pre() == "1.2.3"
        assert vr.build == "build2"
        assert vr.stable() == "1.Y.Z"

        v1 = Version("0.2.3-alpha1+build2")
        vr = _VersionRepr(v1)
        assert vr.major() == "0.Y.Z"
        assert vr.minor() == "0.2.Z"
        assert vr.patch() == "0.2.3"
        assert vr.pre() == "0.2.3-alpha1"
        assert vr.build == "build2"
        assert vr.stable() == "0.2.3-alpha1+build2"

        v1 = Version("+build2")
        vr = _VersionRepr(v1)
        assert vr.major() == ""

    def test_build(self):
        v1 = Version("0.2.3-alpha1+build2")
        vr = _VersionRepr(v1)
        assert vr.build == "build2"
        v2 = Version("0.2.3+b178")
        vr = _VersionRepr(v2)
        assert vr.build == "b178"



import os

import pytest

from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.tools import TestClient


@pytest.mark.parametrize("requires", ["requires", "tool_requires"])
def test_conanfile_txt_deps_ranges(requires):
    """
    conanfile.txt locking it dependencies (with version ranges) using alias
    """
    client = TestClient(light=True)
    client.save({"pkg/conanfile.py": GenConanfile("pkg"),
                 "consumer/conanfile.txt": f"[{requires}]\npkg/(latest)"})
    client.run("create pkg --version=0.1")
    client.run("create pkg --version=0.2")
    with client.chdir("alias"):
        client.run("new alias -d name=pkg -d version=latest -d target=0.1")
        client.run("export .")
    client.run("lock create consumer/conanfile.txt")
    assert "pkg/0.1" in client.out
    assert '"pkg/latest": "pkg/0.1"' in client.load("consumer/conan.lock")

    # Change the alias
    with client.chdir("alias"):
        client.run("new alias -d name=pkg -d version=latest -d target=0.2 -f")
        client.run("export .")
    client.run("install consumer/conanfile.txt")  # use conan.lock by default
    assert "pkg/0.1" in client.out
    assert "pkg/0.2" not in client.out

    os.remove(os.path.join(client.current_folder, "consumer/conan.lock"))
    client.run("install consumer/conanfile.txt")
    assert "pkg/0.2" in client.out
    assert "pkg/0.1" not in client.out


@pytest.mark.parametrize("requires", ["requires", "tool_requires"])
def test_conanfile_txt_deps_ranges_lock_revisions(requires):
    """
    conanfile.txt locking it dependencies (with version ranges)
    """
    client = TestClient(light=True)
    client.save({"pkg/conanfile.py": GenConanfile("pkg"),
                 "consumer/conanfile.txt": f"[{requires}]\npkg/(latest)"})
    client.run("create pkg --version=0.1")
    client.assert_listed_require({"pkg/0.1#a9ec2e5fbb166568d4670a9cd1ef4b26": "Cache"})
    client.run("create pkg --version=0.2")
    with client.chdir("alias"):
        client.run("new alias -d name=pkg -d version=latest -d target=0.1")
        client.run("export .")
    client.run("lock create consumer/conanfile.txt")
    assert "pkg/0.1#a9ec2e5fbb166568d4670a9cd1ef4b26" in client.out
    assert '"pkg/latest": "pkg/0.1"' in client.load("consumer/conan.lock")

    # Create a new revision
    client.save({"pkg/conanfile.py": GenConanfile("pkg").with_class_attribute("potato=42")})
    client.run("create pkg --version=0.1")
    client.assert_listed_require({"pkg/0.1#8d60cd02b0b4aa8fe8b3cae32944c61b": "Cache"})
    client.run("install consumer/conanfile.txt")  # use conan.lock by default
    assert "pkg/0.1#a9ec2e5fbb166568d4670a9cd1ef4b26" in client.out
    assert "pkg/0.1#8d60cd02b0b4aa8fe8b3cae32944c61b" not in client.out

    os.remove(os.path.join(client.current_folder, "consumer/conan.lock"))
    client.run("install consumer/conanfile.txt")
    assert "pkg/0.1#a9ec2e5fbb166568d4670a9cd1ef4b26" not in client.out
    assert "pkg/0.1#8d60cd02b0b4aa8fe8b3cae32944c61b" in client.out



import pytest
from textwrap import dedent

from conan.test.utils.tools import TestClient


class TestCppStdMinimumVersion:

    CONANFILE = dedent("""
        import os
        from conan import ConanFile
        from conan.tools.build import check_min_cppstd, valid_min_cppstd

        class Fake(ConanFile):
            name = "fake"
            version = "0.1"
            settings = "compiler"

            def validate(self):
                check_min_cppstd(self, "17", False)
                self.output.info("valid standard")
                assert valid_min_cppstd(self, "17", False)
        """)

    PROFILE = dedent("""
        [settings]
        compiler=gcc
        compiler.version=9
        compiler.libcxx=libstdc++
        {}
        """)

    @pytest.fixture(autouse=True)
    def setup(self):
        self.client = TestClient()
        self.client.save({"conanfile.py": TestCppStdMinimumVersion.CONANFILE})

    @pytest.mark.parametrize("cppstd", ["17", "gnu17"])
    def test_cppstd_from_settings(self, cppstd):
        profile = TestCppStdMinimumVersion.PROFILE.replace("{}", "compiler.cppstd=%s" % cppstd)
        self.client.save({"myprofile":  profile})
        self.client.run("create . --user=user --channel=channel -pr myprofile")
        assert "valid standard" in self.client.out

    @pytest.mark.parametrize("cppstd", ["11", "gnu11"])
    def test_invalid_cppstd_from_settings(self, cppstd):
        profile = TestCppStdMinimumVersion.PROFILE.replace("{}", "compiler.cppstd=%s" % cppstd)
        self.client.save({"myprofile": profile})
        self.client.run("create . --user=user --channel=channel -pr myprofile", assert_error=True)
        assert "Invalid: Current cppstd (%s) is lower than the required C++ standard (17)." \
                      % cppstd in self.client.out


def test_header_only_check_min_cppstd():
    """
    Check that for a header only package you can check self.info in the validate
    Related to: https://github.com/conan-io/conan/issues/11786
    """
    conanfile = dedent("""
        import os
        from conan import ConanFile
        from conan.tools.build import check_min_cppstd

        class Fake(ConanFile):
            name = "fake"
            version = "0.1"
            settings = "compiler"
            def package_id(self):
                self.info.clear()
            def validate(self):
                check_min_cppstd(self, "11")
        """)
    client = TestClient()
    client.save({"conanfile.py": conanfile})
    client.run("create . -s compiler.cppstd=14")
    client.run("install --require=fake/0.1@ -s compiler.cppstd=14")
    assert "fake/0.1: Already installed!" in client.out


def test_validate_build_check_min_cppstd():
    """
    Check the case that a package needs certain cppstd to build but can be consumed with a lower
    cppstd or even not cppstd defined at all
    """
    conanfile = dedent("""
        import os
        from conan import ConanFile
        from conan.tools.build import check_min_cppstd

        class Fake(ConanFile):
            name = "fake"
            version = "0.1"
            settings = "compiler"
            def validate_build(self):
                check_min_cppstd(self, "17")
            def validate(self):
                print("validated")
            def build(self):
                print("built")
            def package_id(self):
                del self.info.settings.compiler.cppstd
        """)
    client = TestClient()
    client.save({"conanfile.py": conanfile})
    client.run("create . -s compiler.cppstd=14", assert_error=True)
    assert "fake/0.1: Cannot build for this configuration: " \
           "Current cppstd (14) is lower than the required C++ standard (17)." in client.out
    client.run("create . -s compiler.cppstd=17")
    client.run("install --require=fake/0.1@")
    assert "fake/0.1: Already installed!" in client.out
    assert "validated" in client.out



import glob
import os
import textwrap

import pytest

from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.tools import TestClient
from conan.internal.util.files import load


def get_requires_from_content(content):
    for line in content.splitlines():
        if "Requires:" in line:
            return line
    return ""


def test_pkg_config_dirs():
    # https://github.com/conan-io/conan/issues/2756
    conanfile = textwrap.dedent("""
        import os
        from conan import ConanFile

        class PkgConfigConan(ConanFile):
            name = "mylib"
            version = "0.1"

            def package_info(self):
                self.cpp_info.frameworkdirs = []
                self.cpp_info.filter_empty = False
                libname = "mylib"
                fake_dir = os.path.join("/", "my_absoulte_path", "fake")
                include_dir = os.path.join(fake_dir, libname, "include")
                lib_dir = os.path.join(fake_dir, libname, "lib")
                lib_dir1 = os.path.join(self.package_folder, "lib2")
                self.cpp_info.includedirs = [include_dir]
                self.cpp_info.libdirs = [lib_dir, lib_dir1]
                self.cpp_info.bindirs = ["mybin"]
        """)
    client = TestClient()
    client.save({"conanfile.py": conanfile})
    client.run("create .")
    client.run("install --requires=mylib/0.1@ -g PkgConfigDeps")

    pc_path = os.path.join(client.current_folder, "mylib.pc")
    assert os.path.exists(pc_path) is True
    pc_content = load(pc_path)
    assert 'Name: mylib' in pc_content
    assert 'Description: Conan package: mylib' in pc_content
    assert 'Version: 0.1' in pc_content
    assert 'Libs: -L"${libdir}" -L"${libdir1}"' in pc_content
    assert 'Cflags: -I"${includedir}"' in pc_content
    # https://github.com/conan-io/conan/pull/13623
    assert 'bindir=${prefix}/mybin' in pc_content

    def assert_is_abs(path):
        assert os.path.isabs(path) is True

    for line in pc_content.splitlines():
        if line.startswith("includedir="):
            assert_is_abs(line[len("includedir="):])
            assert line.endswith("include")
        elif line.startswith("libdir="):
            assert_is_abs(line[len("libdir="):])
            assert line.endswith("lib")
        elif line.startswith("libdir1="):
            assert "${prefix}/lib2" in line


def test_empty_dirs():
    # Adding in package_info all the empty directories
    conanfile = textwrap.dedent("""
        import os
        from conan import ConanFile

        class PkgConfigConan(ConanFile):
            name = "mylib"
            version = "0.1"

            def package_info(self):
                self.cpp_info.includedirs = []
                self.cpp_info.libdirs = []
                self.cpp_info.bindirs = []
                self.cpp_info.libs = []
                self.cpp_info.frameworkdirs = []
        """)
    client = TestClient()
    client.save({"conanfile.py": conanfile})
    client.run("create .")
    client.run("install --requires=mylib/0.1@ -g PkgConfigDeps")

    pc_path = os.path.join(client.current_folder, "mylib.pc")
    assert os.path.exists(pc_path) is True
    pc_content = load(pc_path)
    expected = textwrap.dedent("""
        Name: mylib
        Description: Conan package: mylib
        Version: 0.1""")
    assert "\n".join(pc_content.splitlines()[1:]) == expected


def test_system_libs():
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.files import save
        import os

        class PkgConfigConan(ConanFile):
            name = "mylib"
            version = "0.1"

            def package(self):
                save(self, os.path.join(self.package_folder, "lib", "file"), "")

            def package_info(self):
                self.cpp_info.libs = ["mylib1", "mylib2"]
                self.cpp_info.system_libs = ["system_lib1", "system_lib2"]
        """)
    client = TestClient()
    client.save({"conanfile.py": conanfile})
    client.run("create .")
    client.run("install --requires=mylib/0.1@ -g PkgConfigDeps")

    pc_content = client.load("mylib.pc")
    assert 'Libs: -L"${libdir}" -lmylib1 -lmylib2 -lsystem_lib1 -lsystem_lib2' in pc_content


def test_multiple_include():
    # https://github.com/conan-io/conan/issues/7056
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.files import save
        import os

        class PkgConfigConan(ConanFile):
            def package(self):
                for p in ["inc1", "inc2", "inc3/foo", "lib1", "lib2"]:
                    save(self, os.path.join(self.package_folder, p, "file"), "")

            def package_info(self):
                self.cpp_info.includedirs = ["inc1", "inc2", "inc3/foo"]
                self.cpp_info.libdirs = ["lib1", "lib2"]
        """)
    client = TestClient()
    client.save({"conanfile.py": conanfile})
    client.run("create . --name=pkg --version=0.1")
    client.run("install --requires=pkg/0.1@ -g PkgConfigDeps")

    pc_content = client.load("pkg.pc")
    assert "includedir=${prefix}/inc1" in pc_content
    assert "includedir1=${prefix}/inc2" in pc_content
    assert "includedir2=${prefix}/inc3/foo" in pc_content
    assert "libdir=${prefix}/lib1" in pc_content
    assert "libdir1=${prefix}/lib2" in pc_content
    assert 'Libs: -L"${libdir}" -L"${libdir1}"' in pc_content
    assert 'Cflags: -I"${includedir}" -I"${includedir1}" -I"${includedir2}"' in pc_content


def test_custom_content():
    # https://github.com/conan-io/conan/issues/7661
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.files import save
        import os
        import textwrap

        class PkgConfigConan(ConanFile):

            def package(self):
                save(self, os.path.join(self.package_folder, "include" ,"file"), "")
                save(self, os.path.join(self.package_folder, "lib" ,"file"), "")

            def package_info(self):
                custom_content = textwrap.dedent(\"""
                        bindir=${prefix}/my/bin/folder
                        fakelibdir=${prefix}/my/lib/folder
                        datadir=${prefix}/share
                        schemasdir=${datadir}/mylib/schemas
                    \""")
                self.cpp_info.set_property("pkg_config_custom_content", custom_content)
                self.cpp_info.includedirs = ["include"]
                self.cpp_info.libdirs = ["lib"]
        """)
    client = TestClient()
    client.save({"conanfile.py": conanfile})
    client.run("create . --name=pkg --version=0.1")
    client.run("install --requires=pkg/0.1@ -g PkgConfigDeps")

    pc_content = client.load("pkg.pc")
    prefix = pc_content.splitlines()[0]
    expected = textwrap.dedent(f"""\
    {prefix}
    libdir=${{prefix}}/lib
    includedir=${{prefix}}/include
    bindir=${{prefix}}/my/bin/folder
    fakelibdir=${{prefix}}/my/lib/folder
    datadir=${{prefix}}/share
    schemasdir=${{datadir}}/mylib/schemas

    Name: pkg
    Description: Conan package: pkg
    Version: 0.1
    Libs: -L"${{libdir}}"
    Cflags: -I"${{includedir}}"
    """)
    assert expected == pc_content


def test_custom_content_and_version_components():
    conanfile = textwrap.dedent("""
        from conan import ConanFile

        class PkgConfigConan(ConanFile):

            def package_info(self):
                self.cpp_info.components["mycomponent"].set_property("pkg_config_custom_content",
                                                                     "componentdir=${prefix}/mydir")
                self.cpp_info.components["mycomponent"].set_property("component_version",
                                                                     "19.8.199")
        """)
    client = TestClient()
    client.save({"conanfile.py": conanfile})
    client.run("create . --name=pkg --version=0.1")
    client.run("install --requires=pkg/0.1@ -g PkgConfigDeps")
    pc_content = client.load("pkg-mycomponent.pc")
    assert "componentdir=${prefix}/mydir" in pc_content
    assert "Version: 19.8.199" in pc_content

    # Now with lockfile
    # https://github.com/conan-io/conan/issues/16197
    lockfile = textwrap.dedent("""
        {
            "version": "0.5",
            "requires": [
                "pkg/0.1#9a5fed2bf506fd28817ddfbc92b07fc1"
            ]
        }
        """)
    client.save({"conan.lock": lockfile})
    client.run("install --requires=pkg/0.1 -g PkgConfigDeps --lockfile=conan.lock")
    pc_content = client.load("pkg-mycomponent.pc")
    assert "componentdir=${prefix}/mydir" in pc_content
    assert "Version: 19.8.199" in pc_content


def test_custom_version():
    # https://github.com/conan-io/conan/issues/16197
    conanfile = textwrap.dedent("""
        from conan import ConanFile

        class PkgConfigConan(ConanFile):
            def package_info(self):
                self.cpp_info.set_property("system_package_version", "19.8.199")
        """)
    client = TestClient()
    client.save({"conanfile.py": conanfile})
    client.run("create . --name=pkg --version=0.1")
    client.run("install --requires=pkg/0.1 -g PkgConfigDeps")

    pc_content = client.load("pkg.pc")
    assert "Version: 19.8.199" in pc_content

    # Now with lockfile
    # https://github.com/conan-io/conan/issues/16197
    lockfile = textwrap.dedent("""
        {
            "version": "0.5",
            "requires": [
                "pkg/0.1#0fe93a852dd6a177bca87cb2d4491a18"
            ]
        }
        """)
    client.save({"conan.lock": lockfile})
    client.run("install --requires=pkg/0.1 -g PkgConfigDeps --lockfile=conan.lock")
    pc_content = client.load("pkg.pc")
    assert "Version: 19.8.199" in pc_content


def test_pkg_with_public_deps_and_component_requires():
    """
    Testing a complex structure like:

    * first/0.1
        - Global pkg_config_name == "myfirstlib"
        - Components: "cmp1"
    * other/0.1
    * second/0.1
        - Requires: "first/0.1"
        - Components: "mycomponent", "myfirstcomp"
            + "mycomponent" requires "first::cmp1"
            + "myfirstcomp" requires "mycomponent"
    * third/0.1
        - Requires: "second/0.1", "other/0.1"

    Expected file structure after running PkgConfigDeps as generator:
        - other.pc
        - myfirstlib-cmp1.pc
        - myfirstlib.pc
        - second-mycomponent.pc
        - second-myfirstcomp.pc
        - second.pc
        - third.pc
    """
    client = TestClient()
    conanfile = textwrap.dedent("""
        from conan import ConanFile

        class Recipe(ConanFile):

            def package_info(self):
                self.cpp_info.set_property("pkg_config_name", "myfirstlib")
                self.cpp_info.components["cmp1"].libs = ["libcmp1"]
    """)
    client.save({"conanfile.py": conanfile})
    client.run("create . --name=first --version=0.1")
    client.save({"conanfile.py": GenConanfile("other", "0.1").with_package_file("file.h", "0.1")})
    client.run("create .")

    conanfile = textwrap.dedent("""
        from conan import ConanFile

        class PkgConfigConan(ConanFile):
            requires = "first/0.1"

            def package_info(self):
                self.cpp_info.components["mycomponent"].requires.append("first::cmp1")
                self.cpp_info.components["myfirstcomp"].requires.append("mycomponent")

        """)
    client.save({"conanfile.py": conanfile}, clean_first=True)
    client.run("create . --name=second --version=0.1")
    client.save({"conanfile.py": GenConanfile("third", "0.1").with_package_file("file.h", "0.1")
                                                             .with_require("second/0.1")
                                                             .with_require("other/0.1")},
                clean_first=True)
    client.run("create .")

    client2 = TestClient(cache_folder=client.cache_folder)
    conanfile = textwrap.dedent("""
        [requires]
        third/0.1

        [generators]
        PkgConfigDeps
        """)
    client2.save({"conanfile.txt": conanfile})
    client2.run("install .")

    pc_content = client2.load("third.pc")
    # Originally posted: https://github.com/conan-io/conan/issues/9939
    assert "Requires: second other" == get_requires_from_content(pc_content)
    pc_content = client2.load("second.pc")
    assert "Requires: second-mycomponent second-myfirstcomp" == get_requires_from_content(pc_content)
    pc_content = client2.load("second-mycomponent.pc")
    assert "Requires: myfirstlib-cmp1" == get_requires_from_content(pc_content)
    pc_content = client2.load("second-myfirstcomp.pc")
    assert "Requires: second-mycomponent" == get_requires_from_content(pc_content)
    pc_content = client2.load("myfirstlib.pc")
    assert "Requires: myfirstlib-cmp1" == get_requires_from_content(pc_content)
    pc_content = client2.load("other.pc")
    assert "" == get_requires_from_content(pc_content)


def test_pkg_with_public_deps_and_component_requires_2():
    """
    Testing another complex structure like:

    * other/0.1
        - Global pkg_config_name == "fancy_name"
        - Components: "cmp1", "cmp2", "cmp3"
            + "cmp1" pkg_config_name == "component1" (it shouldn't be affected by "fancy_name")
            + "cmp3" pkg_config_name == "component3" (it shouldn't be affected by "fancy_name")
            + "cmp3" requires "cmp1"
    * pkg/0.1
        - Requires: "other/0.1" -> "other::cmp1"

    Expected file structure after running PkgConfigDeps as generator:
        - component1.pc
        - component3.pc
        - other-cmp2.pc
        - other.pc
        - pkg.pc
    """
    client = TestClient()
    conanfile = textwrap.dedent("""
        from conan import ConanFile

        class Recipe(ConanFile):

            def package_info(self):
                self.cpp_info.set_property("pkg_config_name", "fancy_name")
                self.cpp_info.components["cmp1"].libs = ["other_cmp1"]
                self.cpp_info.components["cmp1"].set_property("pkg_config_name", "component1")
                self.cpp_info.components["cmp2"].libs = ["other_cmp2"]
                self.cpp_info.components["cmp3"].requires.append("cmp1")
                self.cpp_info.components["cmp3"].set_property("pkg_config_name", "component3")
    """)
    client.save({"conanfile.py": conanfile})
    client.run("create . --name=other --version=1.0")

    conanfile = textwrap.dedent("""
        from conan import ConanFile

        class PkgConfigConan(ConanFile):
            requires = "other/1.0"

            def package_info(self):
                self.cpp_info.requires = ["other::cmp1"]
        """)
    client.save({"conanfile.py": conanfile})
    client.run("create . --name=pkg --version=0.1")

    client2 = TestClient(cache_folder=client.cache_folder)
    conanfile = textwrap.dedent("""
        [requires]
        pkg/0.1

        [generators]
        PkgConfigDeps
        """)
    client2.save({"conanfile.txt": conanfile})
    client2.run("install .")
    pc_content = client2.load("pkg.pc")
    assert "Requires: component1" == get_requires_from_content(pc_content)
    pc_content = client2.load("fancy_name.pc")
    assert "Requires: component1 fancy_name-cmp2 component3" == get_requires_from_content(pc_content)
    assert client2.load("component1.pc")
    assert client2.load("fancy_name-cmp2.pc")
    pc_content = client2.load("component3.pc")
    assert "Requires: component1" == get_requires_from_content(pc_content)


def test_pkg_config_name_full_aliases():
    """
    Testing a simpler structure but paying more attention into several aliases.
    Expected file structure after running PkgConfigDeps as generator:
        - compo1.pc
        - compo1_alias.pc
        - pkg_alias1.pc
        - pkg_alias2.pc
        - pkg_other_name.pc
        - second-mycomponent.pc
        - second.pc
    """
    client = TestClient()
    conanfile = textwrap.dedent("""
        import textwrap
        from conan import ConanFile

        class Recipe(ConanFile):

            def package_info(self):
                custom_content = textwrap.dedent(\"""
                datadir=${prefix}/share
                schemasdir=${datadir}/mylib/schemas
                \""")
                self.cpp_info.set_property("pkg_config_name", "pkg_other_name")
                self.cpp_info.set_property("pkg_config_aliases", ["pkg_alias1", "pkg_alias2"])
                # Custom content only added to root pc file -> pkg_other_name.pc
                self.cpp_info.set_property("pkg_config_custom_content", custom_content)
                self.cpp_info.components["cmp1"].libs = ["libcmp1"]
                self.cpp_info.components["cmp1"].set_property("pkg_config_name", "compo1")
                self.cpp_info.components["cmp1"].set_property("pkg_config_aliases", ["compo1_alias"])
    """)
    client.save({"conanfile.py": conanfile})
    client.run("create . --name=first --version=0.3")

    conanfile = textwrap.dedent("""
        from conan import ConanFile

        class PkgConfigConan(ConanFile):
            requires = "first/0.3"

            def package_info(self):
                self.cpp_info.components["mycomponent"].requires.append("first::cmp1")

        """)
    client.save({"conanfile.py": conanfile}, clean_first=True)
    client.run("create . --name=second --version=0.2")

    conanfile = textwrap.dedent("""
        [requires]
        second/0.2

        [generators]
        PkgConfigDeps
        """)
    client.save({"conanfile.txt": conanfile}, clean_first=True)
    client.run("install .")

    pc_content = client.load("compo1.pc")
    prefix = pc_content.splitlines()[0]
    assert "Description: Conan component: compo1" in pc_content
    assert "Requires" not in pc_content

    pc_content = client.load("compo1_alias.pc")
    content = textwrap.dedent(f"""\
    Name: compo1_alias
    Description: Alias compo1_alias for compo1
    Version: 0.3
    Requires: compo1
    """)
    assert content == pc_content

    pc_content = client.load("pkg_other_name.pc")
    content = textwrap.dedent(f"""\
    {prefix}
    libdir=${{prefix}}/lib
    includedir=${{prefix}}/include
    bindir=${{prefix}}/bin
    datadir=${{prefix}}/share
    schemasdir=${{datadir}}/mylib/schemas

    Name: pkg_other_name
    Description: Conan package: pkg_other_name
    Version: 0.3
    Libs: -L"${{libdir}}"
    Cflags: -I"${{includedir}}"
    Requires: compo1
    """)
    assert content == pc_content

    pc_content = client.load("pkg_alias1.pc")
    content = textwrap.dedent(f"""\
    Name: pkg_alias1
    Description: Alias pkg_alias1 for pkg_other_name
    Version: 0.3
    Requires: pkg_other_name
    """)
    assert content == pc_content

    pc_content = client.load("pkg_alias2.pc")
    content = textwrap.dedent(f"""\
    Name: pkg_alias2
    Description: Alias pkg_alias2 for pkg_other_name
    Version: 0.3
    Requires: pkg_other_name
    """)
    assert content == pc_content

    pc_content = client.load("second-mycomponent.pc")
    assert "Requires: compo1" == get_requires_from_content(pc_content)


def test_components_and_package_pc_creation_order():
    """
    Testing if the root package PC file name matches with any of the components one, the first one
    is not going to be created. Components have more priority than root package.

    Issue related: https://github.com/conan-io/conan/issues/10341
    """
    client = TestClient()
    conanfile = textwrap.dedent("""
        from conan import ConanFile

        class PkgConfigConan(ConanFile):

            def package_info(self):
                self.cpp_info.set_property("pkg_config_name", "OpenCL")
                self.cpp_info.components["_opencl-headers"].set_property("pkg_config_name", "OpenCL")
                self.cpp_info.components["_opencl-other"].set_property("pkg_config_name", "OtherCL")
        """)
    client.save({"conanfile.py": conanfile})
    client.run("create . --name=opencl --version=1.0")

    conanfile = textwrap.dedent("""
        from conan import ConanFile

        class PkgConfigConan(ConanFile):
            requires = "opencl/1.0"

            def package_info(self):
                self.cpp_info.components["comp"].set_property("pkg_config_name", "pkgb")
                self.cpp_info.components["comp"].requires.append("opencl::_opencl-headers")
        """)
    client.save({"conanfile.py": conanfile}, clean_first=True)
    client.run("create . --name=pkgb --version=1.0")

    conanfile = textwrap.dedent("""
        [requires]
        pkgb/1.0

        [generators]
        PkgConfigDeps
        """)
    client.save({"conanfile.txt": conanfile}, clean_first=True)
    client.run("install .")
    pc_files = [os.path.basename(i) for i in glob.glob(os.path.join(client.current_folder, '*.pc'))]
    pc_files.sort()
    # Let's check all the PC file names created just in case
    assert pc_files == ['OpenCL.pc', 'OtherCL.pc', 'pkgb.pc']
    pc_content = client.load("OpenCL.pc")
    assert "Name: OpenCL" in pc_content
    assert "Description: Conan component: OpenCL" in pc_content
    assert "Requires:" not in pc_content
    pc_content = client.load("pkgb.pc")
    assert "Requires: OpenCL" in get_requires_from_content(pc_content)


def test_pkgconfigdeps_with_test_requires():
    """
    PkgConfigDeps has to create any test requires declared on the recipe.

    Related issue: https://github.com/conan-io/conan/issues/11376
    """
    client = TestClient()
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        class Pkg(ConanFile):
            def package_info(self):
                self.cpp_info.libs = ["lib%s"]
        """)
    with client.chdir("app"):
        client.save({"conanfile.py": conanfile % "app"})
        client.run("create . --name=app --version=1.0")
    with client.chdir("test"):
        client.save({"conanfile.py": conanfile % "test"})
        client.run("create . --name=test --version=1.0")
    # Create library having build and test requires
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        class HelloLib(ConanFile):
            def build_requirements(self):
                self.test_requires('app/1.0')
                self.test_requires('test/1.0')
        """)
    client.save({"conanfile.py": conanfile}, clean_first=True)
    client.run("install . -g PkgConfigDeps")
    assert "Description: Conan package: test" in client.load("test.pc")
    assert "Description: Conan package: app" in client.load("app.pc")


def test_with_editable_layout():
    """
    https://github.com/conan-io/conan/issues/11435
    """
    client = TestClient()
    dep = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.files import save
        class Dep(ConanFile):
            name = "dep"
            version = "0.1"
            def layout(self):
                self.cpp.source.includedirs = ["include"]
            def package_info(self):
                self.cpp_info.libs = ["mylib"]
        """)
    client.save({"dep/conanfile.py": dep,
                 "dep/include/header.h": "",
                 "pkg/conanfile.py": GenConanfile("pkg", "0.1").with_requires("dep/0.1")})
    client.run("create dep")
    client.run("editable add dep")
    with client.chdir("pkg"):
        client.run("install . -g PkgConfigDeps")
        pc = client.load("dep.pc")
        assert 'Libs: -L"${libdir}" -lmylib' in pc
        assert 'includedir=' in pc
        assert 'Cflags: -I"${includedir}"' in pc


def test_tool_requires():
    """
    Testing if PC files are created for tool requires if build_context_activated/_suffix is used.

    Issue related: https://github.com/conan-io/conan/issues/11710
    """
    client = TestClient()
    conanfile = textwrap.dedent("""
        from conan import ConanFile

        class PkgConfigConan(ConanFile):

            def package_info(self):
                self.cpp_info.libs = ["libtool"]
        """)
    client.save({"conanfile.py": conanfile})
    client.run("create . --name tool --version 1.0")

    conanfile = textwrap.dedent("""
        from conan import ConanFile

        class PkgConfigConan(ConanFile):

            def package_info(self):
                self.cpp_info.set_property("pkg_config_name", "libother")
                self.cpp_info.components["cmp1"].libs = ["other_cmp1"]
                self.cpp_info.components["cmp1"].set_property("pkg_config_name", "component1")
                self.cpp_info.components["cmp2"].libs = ["other_cmp2"]
                self.cpp_info.components["cmp3"].requires.append("cmp1")
                self.cpp_info.components["cmp3"].set_property("pkg_config_name", "component3")
        """)
    client.save({"conanfile.py": conanfile}, clean_first=True)
    client.run("create . --name other --version 1.0")

    conanfile = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.gnu import PkgConfigDeps

        class PkgConfigConan(ConanFile):
            name = "demo"
            version = "1.0"

            def build_requirements(self):
                self.build_requires("tool/1.0")
                self.build_requires("other/1.0")

            def generate(self):
                tc = PkgConfigDeps(self)
                tc.build_context_activated = ["other", "tool"]
                tc.build_context_suffix = {"tool": "_bt", "other": "_bo"}
                tc.generate()
        """)
    client.save({"conanfile.py": conanfile}, clean_first=True)
    client.run("install . -pr:h default -pr:b default")
    pc_files = [os.path.basename(i) for i in glob.glob(os.path.join(client.current_folder, '*.pc'))]
    pc_files.sort()
    # Let's check all the PC file names created just in case
    assert pc_files == ['component1_bo.pc', 'component3_bo.pc',
                        'libother_bo-cmp2.pc', 'libother_bo.pc', 'tool_bt.pc']
    pc_content = client.load("tool_bt.pc")
    assert "Name: tool_bt" in pc_content
    pc_content = client.load("libother_bo.pc")
    assert "Name: libother_bo" in pc_content
    assert "Requires: component1_bo libother_bo-cmp2 component3_bo" == get_requires_from_content(pc_content)
    pc_content = client.load("component1_bo.pc")
    assert "Name: component1_bo" in pc_content
    pc_content = client.load("libother_bo-cmp2.pc")
    assert "Name: libother_bo-cmp2" in pc_content
    pc_content = client.load("component3_bo.pc")
    assert "Name: component3_bo" in pc_content
    assert "Requires: component1_bo" == get_requires_from_content(pc_content)


def test_tool_requires_not_created_if_no_activated():
    """
    Testing if there are no PC files created in no context are activated
    """
    client = TestClient()
    conanfile = textwrap.dedent("""
        from conan import ConanFile

        class PkgConfigConan(ConanFile):

            def package_info(self):
                self.cpp_info.libs = ["libtool"]
        """)
    client.save({"conanfile.py": conanfile})
    client.run("create . --name tool --version 1.0")

    conanfile = textwrap.dedent("""
        from conan import ConanFile

        class PkgConfigConan(ConanFile):
            name = "demo"
            version = "1.0"
            generators = "PkgConfigDeps"

            def build_requirements(self):
                self.build_requires("tool/1.0")

        """)
    client.save({"conanfile.py": conanfile}, clean_first=True)
    client.run("install . -pr:h default -pr:b default")
    pc_files = [os.path.basename(i) for i in glob.glob(os.path.join(client.current_folder, '*.pc'))]
    pc_files.sort()
    assert pc_files == []


def test_tool_requires_error_if_no_build_suffix():
    """
    Testing if same dependency exists in both require and build require (without suffix)
    """
    client = TestClient()
    conanfile = textwrap.dedent("""
        from conan import ConanFile

        class PkgConfigConan(ConanFile):

            def package_info(self):
                self.cpp_info.libs = ["libtool"]
        """)
    client.save({"conanfile.py": conanfile})
    client.run("create . --name tool --version 1.0")

    conanfile = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.gnu import PkgConfigDeps

        class PkgConfigConan(ConanFile):
            name = "demo"
            version = "1.0"

            def requirements(self):
                self.requires("tool/1.0")

            def build_requirements(self):
                self.build_requires("tool/1.0")

            def generate(self):
                tc = PkgConfigDeps(self)
                tc.build_context_activated = ["tool"]
                tc.generate()
        """)
    client.save({"conanfile.py": conanfile}, clean_first=True)
    client.run("install . -pr:h default -pr:b default", assert_error=True)
    assert "The packages ['tool'] exist both as 'require' and as 'build require'" in client.out


def test_error_missing_pc_build_context():
    """
    PkgConfigDeps was failing, not generating the zlib.pc in the
    build context, for a test_package that both requires(example/1.0) and
    tool_requires(example/1.0), which depends on zlib
    # https://github.com/conan-io/conan/issues/12664
    """
    c = TestClient()
    example = textwrap.dedent("""
        import os
        from conan import ConanFile
        class Example(ConanFile):
            name = "example"
            version = "1.0"
            requires = "game/1.0"
            generators = "PkgConfigDeps"
            settings = "build_type"
            def build(self):
                assert os.path.exists("math.pc")
                assert os.path.exists("engine.pc")
                assert os.path.exists("game.pc")
            """)
    c.save({"math/conanfile.py": GenConanfile("math", "1.0").with_settings("build_type"),
            "engine/conanfile.py": GenConanfile("engine", "1.0").with_settings("build_type")
                                                                .with_require("math/1.0"),
            "game/conanfile.py": GenConanfile("game", "1.0").with_settings("build_type")
                                                            .with_requires("engine/1.0"),
            "example/conanfile.py": example,
            # With ``with_test()`` it already generates a requires(example/1.0)
            "example/test_package/conanfile.py": GenConanfile().with_build_requires("example/1.0")
                                                               .with_test("pass")})
    c.run("create math")
    c.run("create engine")
    c.run("create game")
    # This used to crash because of the assert inside the build() method
    c.run("create example -pr:b=default -pr:h=default")
    # Now make sure we can actually build with build!=host context
    # The debug binaries are missing, so adding --build=missing
    c.run("create example -pr:b=default -pr:h=default -s:h build_type=Debug --build=missing "
          "--build=example")

    c.assert_listed_require({"example/1.0": "Cache"})
    c.assert_listed_require({"example/1.0": "Cache"}, build=True)


class TestPCGenerationBuildContext:
    """
    https://github.com/conan-io/conan/issues/14920
    """
    def test_pc_generate(self):
        c = TestClient()
        tool = textwrap.dedent("""
            import os
            from conan import ConanFile
            from conan.tools.gnu import PkgConfigDeps

            class Example(ConanFile):
                name = "tool"
                version = "1.0"
                requires = "wayland/1.0"
                tool_requires = "wayland/1.0"

                def generate(self):
                    deps = PkgConfigDeps(self)
                    deps.build_context_activated = ["wayland", "dep"]
                    deps.build_context_suffix = {"wayland": "_BUILD", "dep": "_BUILD"}
                    deps.generate()

                def build(self):
                    assert os.path.exists("wayland.pc")
                    assert os.path.exists("wayland_BUILD.pc")
                    assert os.path.exists("dep.pc")
                    assert os.path.exists("dep_BUILD.pc")
                """)
        c.save({"dep/conanfile.py": GenConanfile("dep", "1.0").with_package_type("shared-library"),
                "wayland/conanfile.py": GenConanfile("wayland", "1.0").with_requires("dep/1.0"),
                "tool/conanfile.py": tool,
                "app/conanfile.py": GenConanfile().with_tool_requires("tool/1.0")})
        c.run("export dep")
        c.run("export wayland")
        c.run("export tool")
        c.run("install app --build=missing")
        assert "Install finished successfully" in c.out  # the asserts in build() didn't fail
        # Deprecation warning!
        assert "PkgConfigDeps.build_context_suffix attribute has been deprecated" in c.out
        # Now make sure we can actually build with build!=host context
        c.run("install app -s:h build_type=Debug --build=missing")
        assert "Install finished successfully" in c.out  # the asserts in build() didn't fail

    def test_pc_generate_components(self):
        c = TestClient()
        tool = textwrap.dedent("""
            import os
            from conan import ConanFile
            from conan.tools.gnu import PkgConfigDeps

            class Example(ConanFile):
                name = "tool"
                version = "1.0"
                requires = "wayland/1.0"
                tool_requires = "wayland/1.0"

                def generate(self):
                    deps = PkgConfigDeps(self)
                    deps.build_context_activated = ["wayland", "dep"]
                    deps.build_context_suffix = {"wayland": "_BUILD", "dep": "_BUILD"}
                    deps.generate()

                def build(self):
                    assert os.path.exists("wayland.pc")
                    assert os.path.exists("wayland-client.pc")
                    assert os.path.exists("wayland-server.pc")
                    assert os.path.exists("wayland_BUILD.pc")
                    assert os.path.exists("wayland_BUILD-client.pc")
                    assert os.path.exists("wayland_BUILD-server.pc")
                    assert os.path.exists("dep.pc")
                    assert os.path.exists("dep_BUILD.pc")

                    # Issue: https://github.com/conan-io/conan/issues/12342
                    # Issue: https://github.com/conan-io/conan/issues/14935
                    assert not os.path.exists("build/wayland.pc")
                    assert not os.path.exists("build/wayland-client.pc")
                    assert not os.path.exists("build/wayland-server.pc")
                    assert not os.path.exists("build/dep.pc")
                """)
        wayland = textwrap.dedent("""
            from conan import ConanFile

            class Pkg(ConanFile):
                name = "wayland"
                version = "1.0"
                requires = "dep/1.0"

                def package_info(self):
                    self.cpp_info.components["client"].libs = []
                    self.cpp_info.components["server"].libs = []
            """)
        c.save({"dep/conanfile.py": GenConanfile("dep", "1.0").with_package_type("shared-library"),
                "wayland/conanfile.py": wayland,
                "tool/conanfile.py": tool,
                "app/conanfile.py": GenConanfile().with_tool_requires("tool/1.0")})
        c.run("export dep")
        c.run("export wayland")
        c.run("export tool")
        c.run("install app --build=missing")
        assert "Install finished successfully" in c.out  # the asserts in build() didn't fail
        # Now make sure we can actually build with build!=host context
        c.run("install app -s:h build_type=Debug --build=missing")
        assert "Install finished successfully" in c.out  # the asserts in build() didn't fail

    @pytest.mark.parametrize("build_folder_name", ["build", ""])
    def test_pc_generate_components_in_build_context_folder(self, build_folder_name):
        c = TestClient()
        tool = textwrap.dedent("""
            import os
            from conan import ConanFile
            from conan.tools.gnu import PkgConfigDeps

            class Example(ConanFile):
                name = "tool"
                version = "1.0"
                requires = "wayland/1.0"
                tool_requires = "wayland/1.0"

                def generate(self):
                    deps = PkgConfigDeps(self)
                    deps.build_context_activated = ["wayland", "dep"]
                    deps.build_context_folder = "{build_folder_name}"
                    deps.generate()

                def build(self):
                    assert os.path.exists("wayland.pc")
                    assert os.path.exists("wayland-client.pc")
                    assert os.path.exists("wayland-server.pc")
                    assert os.path.exists("dep.pc")

                    # Issue: https://github.com/conan-io/conan/issues/12342
                    # Issue: https://github.com/conan-io/conan/issues/14935
                    if "{build_folder_name}":
                        assert os.path.exists("{build_folder_name}/wayland.pc")
                        assert os.path.exists("{build_folder_name}/wayland-client.pc")
                        assert os.path.exists("{build_folder_name}/wayland-server.pc")
                        assert os.path.exists("{build_folder_name}/dep.pc")
                """.format(build_folder_name=build_folder_name))
        wayland = textwrap.dedent("""
            from conan import ConanFile

            class Pkg(ConanFile):
                name = "wayland"
                version = "1.0"
                requires = "dep/1.0"

                def package_info(self):
                    self.cpp_info.components["client"].libs = []
                    self.cpp_info.components["server"].libs = []
            """)
        c.save({"dep/conanfile.py": GenConanfile("dep", "1.0").with_package_type("shared-library"),
                "wayland/conanfile.py": wayland,
                "tool/conanfile.py": tool,
                "app/conanfile.py": GenConanfile().with_tool_requires("tool/1.0")})
        c.run("export dep")
        c.run("export wayland")
        c.run("export tool")
        c.run("install app --build=missing")
        assert "Install finished successfully" in c.out  # the asserts in build() didn't fail
        # Now make sure we can actually build with build!=host context
        c.run("install app -s:h build_type=Debug --build=missing")
        assert "Install finished successfully" in c.out  # the asserts in build() didn't fail

    @pytest.mark.parametrize("build_folder_name", ["build", ""])
    def test_pkg_config_deps_set_in_build_context_folder(self, build_folder_name):
        c = TestClient()
        tool = textwrap.dedent("""
                    import os
                    from conan import ConanFile
                    from conan.tools.gnu import PkgConfigDeps

                    class Example(ConanFile):
                       name = "tool"
                       version = "1.0"
                       requires = "wayland/1.0"
                       tool_requires = "wayland/1.0"

                    def generate(self):
                       deps = PkgConfigDeps(self)
                       deps.set_property("wayland", "pkg_config_name", "waylandx264")
                       deps.build_context_activated = ["wayland", "dep"]
                       deps.build_context_folder = "{build_folder_name}"
                       deps.generate()

                    def build(self):
                        assert os.path.exists("waylandx264.pc")
                        assert not os.path.exists("wayland.pc")
                        if "{build_folder_name}":
                            assert os.path.exists("{build_folder_name}/waylandx264.pc")
                            assert not os.path.exists("{build_folder_name}/wayland.pc")
                   """.format(build_folder_name=build_folder_name))
        wayland = textwrap.dedent("""
               from conan import ConanFile

               class Pkg(ConanFile):
                   name = "wayland"
                   version = "1.0"
                   requires = "dep/1.0"

                   def package_info(self):
                       self.cpp_info.components["client"].libs = []
                       self.cpp_info.components["server"].libs = []
               """)
        c.save({"dep/conanfile.py": GenConanfile("dep", "1.0").with_package_type("shared-library"),
                "wayland/conanfile.py": wayland,
                "tool/conanfile.py": tool,
                "app/conanfile.py": GenConanfile().with_tool_requires("tool/1.0")})
        c.run("create dep")
        c.run("create wayland")
        c.run("create tool")
        c.run("install app --build=missing")
        assert "Install finished successfully" in c.out  # the asserts in build() didn't fail
        # Now make sure we can actually build with build!=host context
        c.run("install app -s:h build_type=Debug --build=missing")
        assert "Install finished successfully" in c.out  # the asserts in build() didn't fail

    def test_tool_requires_error_if_folder_and_suffix(self):
        client = TestClient()
        conanfile = textwrap.dedent("""
            from conan import ConanFile

            class PkgConfigConan(ConanFile):

                def package_info(self):
                    self.cpp_info.libs = ["libtool"]
            """)
        client.save({"conanfile.py": conanfile})
        client.run("create . --name tool --version 1.0")

        conanfile = textwrap.dedent("""
            from conan import ConanFile
            from conan.tools.gnu import PkgConfigDeps

            class PkgConfigConan(ConanFile):
                name = "demo"
                version = "1.0"

                def requirements(self):
                    self.requires("tool/1.0")

                def build_requirements(self):
                    self.build_requires("tool/1.0")

                def generate(self):
                    tc = PkgConfigDeps(self)
                    tc.build_context_activated = ["tool"]
                    tc.build_context_folder = "build"
                    tc.build_context_suffix = {"tool": "_bt"}
                    tc.generate()
            """)
        client.save({"conanfile.py": conanfile}, clean_first=True)
        client.run("install . -pr:h default -pr:b default", assert_error=True)
        assert ("It's not allowed to define both PkgConfigDeps.build_context_folder "
                "and PkgConfigDeps.build_context_suffix (deprecated).") in client.out


def test_pkg_config_deps_and_private_deps():
    """
    Testing that no errors are raised when the dependency tree has a private one in the middle
    of it.

    Issue related: https://github.com/conan-io/conan/issues/15311
    """
    client = TestClient()
    client.save({"conanfile.py": GenConanfile("private", "0.1")})
    client.run("create .")
    conanfile = textwrap.dedent("""
    from conan import ConanFile
    class ConsumerConan(ConanFile):
        settings = "os", "compiler", "build_type", "arch"
        generators = "PkgConfigDeps"
        name = "pkg"
        version = "0.1"
        def requirements(self):
            self.requires("private/0.1", visible=False)
    """)
    client.save({"conanfile.py": conanfile}, clean_first=True)
    client.run("create .")
    conanfile = textwrap.dedent("""
    from conan import ConanFile
    class ConsumerConan(ConanFile):
        settings = "os", "compiler", "build_type", "arch"
        generators = "PkgConfigDeps"

        def requirements(self):
            self.requires("pkg/0.1")
    """)
    client.save({"conanfile.py": conanfile}, clean_first=True)
    # Now, it passes and creates the pc files correctly (the skipped one is not created)
    client.run("install .")
    assert "Requires:" not in client.load("pkg.pc")


def test_using_deployer_folder():
    """
    Testing that the absolute path is kept as the prefix instead of the
    relative path.

    Issue related: https://github.com/conan-io/conan/issues/16543
    """
    client = TestClient()
    client.save({"dep/conanfile.py": GenConanfile("dep", "0.1")})
    client.run("create dep/conanfile.py")
    client.run("install --requires=dep/0.1 --deployer=direct_deploy "
               "--deployer-folder=mydeploy -g PkgConfigDeps")
    content = client.load("dep.pc")
    prefix_base = client.current_folder.replace('\\', '/')
    assert f"prefix={prefix_base}/mydeploy/direct_deploy/dep" in content
    assert "libdir=${prefix}/lib" in content
    assert "includedir=${prefix}/include" in content
    assert "bindir=${prefix}/bin" in content


def test_pkg_config_deps_set_property():
    c = TestClient()
    app = textwrap.dedent("""\
        import os
        from conan import ConanFile
        from conan.tools.gnu import PkgConfigDeps
        class Pkg(ConanFile):
            settings = "build_type"
            requires = "dep/0.1", "other/0.1"
            def generate(self):
                pc = PkgConfigDeps(self)
                pc.set_property("dep", "pkg_config_name", "depx264")
                pc.set_property("other::mycomp1", "nosoname", True)
                pc.set_property("other::mycomp1", "pkg_config_name", "new_other_comp")
                pc.generate()
            """)

    pkg_info = {"components": {"mycomp1": {"libs": ["mylib"]}}}
    c.save({"dep/conanfile.py": GenConanfile("dep", "0.1").with_package_type("shared-library"),
            "other/conanfile.py": GenConanfile("other", "0.1").with_package_type("shared-library")
                                                              .with_package_info(pkg_info, {}),
            "app/conanfile.py": app})
    c.run("create dep")
    c.run("create other")
    c.run("install app")
    assert not os.path.exists(os.path.join(c.current_folder, "app", "dep.pc"))

    dep = c.load("app/depx264.pc")
    assert 'Name: depx264' in dep
    other = c.load("app/other.pc")
    assert 'Name: other' in other
    other_mycomp1 = c.load("app/new_other_comp.pc")
    assert 'Name: new_other_comp' in other_mycomp1
    assert other.split("\n")[0] == other_mycomp1.split("\n")[0]


def test_pkg_with_duplicated_component_requires():
    """
    Testing that even having duplicated component requires, the PC does not include them.
    Issue: https://github.com/conan-io/conan/issues/18283
    """
    client = TestClient()
    conanfile = textwrap.dedent("""
        from conan import ConanFile

        class PkgConfigConan(ConanFile):
            def package_info(self):
                self.cpp_info.components["mycomponent"].libs = []
                self.cpp_info.components["myfirstcomp"].requires.append("mycomponent")
                # Duplicate one
                self.cpp_info.components["myfirstcomp"].requires.append("mycomponent")

        """)
    client.save({"conanfile.py": conanfile}, clean_first=True)
    client.run("create . --name=mylib --version=0.1")
    client.save({"conanfile.py": GenConanfile("pkg", "0.1").with_require("mylib/0.1")},
                clean_first=True)
    client.run("install . -g PkgConfigDeps")
    pc_content = client.load("mylib-myfirstcomp.pc")
    assert "Requires: mylib-mycomponent" == get_requires_from_content(pc_content)


def test_pkg_skip_component():
    conanfile_a = textwrap.dedent("""
        from conan import ConanFile
        class PkgConfigConan(ConanFile):
            name = "pkg_a"
            version = "0.1"
            def package_info(self):
                self.cpp_info.set_property("pkg_config_name", "none")
        """)
    conanfile_b = textwrap.dedent("""
        from conan import ConanFile
        class PkgConfigConan(ConanFile):
            name = "pkg_b"
            version = "0.1"
            requires = "pkg_a/0.1"
            def package_info(self):
                self.cpp_info.components["cmp1"].set_property("pkg_config_name", "b-cmp1")
        """)

    conanfile_c = textwrap.dedent("""
            from conan import ConanFile
            class PkgConfigConan(ConanFile):
                name = "pkg_c"
                version = "0.1"
                requires = "pkg_b/0.1"
                def package_info(self):
                    self.cpp_info.components["cmp2"].set_property("pkg_config_name", "none")
            """)
    tc = TestClient(light=True)
    tc.save({"a/conanfile.py": conanfile_a,
             "b/conanfile.py": conanfile_b,
             "c/conanfile.py": conanfile_c})
    tc.run("create a")
    tc.run("create b")
    tc.run("create c")

    tc.run("install --requires=pkg_c/0.1 --generator=PkgConfigDeps -of=out")
    install_contents = os.listdir(os.path.join(tc.current_folder, "out"))
    assert "pkg_a.pc" not in install_contents
    assert "pkg_b.pc" in install_contents
    pkg_b_content = tc.load(os.path.join("out", "pkg_b.pc"))
    pkg_b_requires = get_requires_from_content(pkg_b_content)
    assert "b-cmp1" in pkg_b_requires
    assert "pkg_a" not in pkg_b_requires
    assert "none" not in pkg_b_requires
    assert "b-cmp1.pc" in install_contents
    b_cmp1_content = tc.load(os.path.join("out", "b-cmp1.pc"))
    assert "Requires:" not in b_cmp1_content
    assert "pkg_c.pc" in install_contents
    # Components can not skip the PC file creation
    assert "none.pc" in install_contents



import platform

import pytest

import textwrap

from conan.test.assets.sources import gen_function_cpp
from conan.test.utils.tools import TestClient
from jinja2 import Template


def gen_qbs_application(**context):
    qbsfile = textwrap.dedent('''
        CppApplication {
            files: [{% for s in files %} "{{s}}", {% endfor %}]
            {% for d in deps %}
            Depends {
                name: "{{d}}"
            }
            {% endfor %}
            qbsModuleProviders: ["conan"]
        }
    ''')

    t = Template(qbsfile)
    return t.render(**context)


@pytest.mark.skipif(platform.system() != "Linux", reason="QBS only for Linux now")
@pytest.mark.tool("qbs")
def test_qbsdeps_with_test_requires_header_only():
    client = TestClient()
    with client.chdir("lib"):
        conanfile = textwrap.dedent('''
            from conan import ConanFile
            from conan.tools.files import copy
            import os
            class Recipe(ConanFile):
                exports_sources = ("hello.h")
                version = '0.1.0'
                name = 'hello'
                def package(self):
                    copy(self,
                         "hello.h",
                         src=self.source_folder,
                         dst=os.path.join(self.package_folder, "include"))
        ''')
        header = textwrap.dedent('''
            #pragma once
            inline int hello() { return 0; }
        ''')
        client.save({
            'conanfile.py': conanfile,
            'hello.h': header
        })
        client.run('create .')

    conanfile = textwrap.dedent('''
        from conan import ConanFile
        class Recipe(ConanFile):
            settings = "os", "compiler", "build_type", "arch"
            generators = "QbsDeps"
            def requirements(self):
                self.requires('hello/0.1.0')
        ''')
    client.save({
        "conanfile.py": conanfile,
        "main.cpp": gen_function_cpp(name="main", includes=["hello"], calls=["hello"]),
        "app.qbs": gen_qbs_application(files=["main.cpp"], deps=["hello"]),
    }, clean_first=True)
    client.run("install .")
    client.run_command("qbs build moduleProviders.conan.installDirectory:.")


@pytest.mark.skipif(platform.system() != "Linux", reason="QBS only for Linux now")
@pytest.mark.tool("qbs")
def test_qbsdeps_with_test_requires_lib():
    client = TestClient()
    with client.chdir("lib"):
        client.run("new qbs_lib -d name=hello -d version=1.0")
        client.run("create . -tf=\"\"")

    conanfile = textwrap.dedent('''
        from conan import ConanFile
        class Recipe(ConanFile):
            settings = "os", "compiler", "build_type", "arch"
            generators = "QbsDeps"
            def requirements(self):
                self.requires('hello/1.0')
        ''')
    client.save({
        "conanfile.py": conanfile,
        "main.cpp": gen_function_cpp(name="main", includes=["hello"], calls=["hello"]),
        "app.qbs": gen_qbs_application(files=["main.cpp"], deps=["hello"]),
    }, clean_first=True)
    client.run("install .")
    client.run_command("qbs build moduleProviders.conan.installDirectory:.")


@pytest.mark.skipif(platform.system() != "Linux", reason="QBS only for Linux now")
@pytest.mark.tool("qbs")
def test_qbsdeps_with_qbs_toolchain():
    client = TestClient()
    with client.chdir("lib"):
        client.run("new qbs_lib -d name=hello -d version=1.0")
        client.run("create . -tf=\"\"")

    conanfile = textwrap.dedent('''
        from conan import ConanFile
        from conan.tools.qbs import Qbs

        class Recipe(ConanFile):
            name = "app"
            version = "1.2"
            exports_sources = "*.cpp", "*.h", "*.qbs"
            settings = "os", "compiler", "build_type", "arch"
            generators = "QbsDeps"

            def requirements(self):
                self.requires("hello/1.0")

            def layout(self):
                self.folders.generators = "generators"

            def build(self):
                qbs = Qbs(self)
                qbs.resolve()
                qbs.build()

            def install(self):
                qbs = Qbs(self)
                qbs.install()
        ''')
    client.save({
        "conanfile.py": conanfile,
        "main.cpp": gen_function_cpp(name="main", includes=["hello"], calls=["hello"]),
        "app.qbs": gen_qbs_application(files=["main.cpp"], deps=["hello"]),
    }, clean_first=True)
    client.run("create .")



import os
import textwrap

from conan.internal.paths import CONANFILE
from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.test_files import temp_folder
from conan.test.utils.tools import NO_SETTINGS_PACKAGE_ID, TestClient
from conan.test.utils.env import environment_update
from conan.internal.util.files import save


class TestDefaultProfile:

    def test_conanfile_txt_incomplete_profile(self):
        conanfile = GenConanfile()

        client = TestClient()

        client.save({CONANFILE: conanfile})
        client.run("create . --name=pkg --version=0.1 --user=lasote --channel=stable")
        assert "pkg/0.1@lasote/stable: Package '%s' created" % NO_SETTINGS_PACKAGE_ID \
                      in client.out

        client.save({"conanfile.txt": "[requires]\npkg/0.1@lasote/stable"}, clean_first=True)
        client.run('install .')
        assert "pkg/0.1@lasote/stable: Already installed!" in client.out

    def test_change_default_profile(self):
        br = '''
import os
from conan import ConanFile
from conan.tools.env import VirtualBuildEnv

class MyConanfile(ConanFile):
    name = "mylib"
    version = "0.1"

    def build(self):
        build_env = VirtualBuildEnv(self).vars()
        with build_env.apply():
            assert(os.environ.get("Value1") == "A")
'''
        tmp = temp_folder()
        default_profile_path = os.path.join(tmp, "myprofile")
        save(default_profile_path, "[buildenv]\nValue1=A")
        client = TestClient()
        client.save_home({"global.conf": "core:default_profile={}".format(default_profile_path)})

        client.save({CONANFILE: br})
        client.run("export . --user=lasote --channel=stable")
        client.run('install --requires=mylib/0.1@lasote/stable --build="*"')

        # Now use a name, in the default profile folder
        os.unlink(default_profile_path)
        save(os.path.join(client.paths.profiles_path, "other"), "[buildenv]\nValue1=A")
        client.save_home({"global.conf": "core:default_profile=other"})
        client.save({CONANFILE: br})
        client.run("export . --user=lasote --channel=stable")
        client.run('install --requires=mylib/0.1@lasote/stable --build="*"')

    def test_profile_applied_ok(self):
        br = textwrap.dedent('''
            from conan import ConanFile

            class BuildRequireConanfile(ConanFile):
                name = "br"
                version = "1.0"

                def package_info(self):
                    self.buildenv_info.define("MyVAR", "from_build_require")
            ''')

        cf = textwrap.dedent('''
            import os, platform
            from conan import ConanFile

            class MyConanfile(ConanFile):
                requires = "br/1.0"
                def build(self):
                    if platform.system() == "Windows":
                        self.run("set MyVAR")
                    else:
                        self.run("printenv MyVAR")
            ''')
        client = TestClient()
        client.save({"br/conanfile.py": br,
                     "consumer/conanfile.py": cf,
                     "profile": "[buildenv]\nMyVAR=23"})
        client.run("create br")
        client.run("build consumer")
        assert "from_build_require" in client.out
        client.run("build consumer -pr=profile")
        assert "23" in client.out
        assert "from_build_require" not in client.out

    def test_env_default_profile(self):
        conanfile = '''
import os
from conan import ConanFile
from conan.tools.env import VirtualBuildEnv

class MyConanfile(ConanFile):

    def build(self):
        build_env = VirtualBuildEnv(self).vars()
        with build_env.apply():
            self.output.info(">>> env_variable={}".format(os.environ.get('env_variable')))
'''

        client = TestClient()
        client.save({CONANFILE: conanfile})

        # Test with the 'default' profile
        env_variable = "env_variable=profile_default"
        client.save_home({"profiles/default": "[settings]\nos=Windows\n[buildenv]\n" + env_variable})
        client.run("create . --name=name --version=version --user=user --channel=channel")
        assert ">>> " + env_variable in client.out

        # Test with a profile set using and environment variable
        tmp = temp_folder()
        env_variable = "env_variable=profile_environment"
        default_profile_path = os.path.join(tmp, 'env_profile')
        save(default_profile_path, "[settings]\nos=Windows\n[buildenv]\n" + env_variable)
        with environment_update({'CONAN_DEFAULT_PROFILE': default_profile_path}):
            client.run("create . --name=name --version=version --user=user --channel=channel")
            assert ">>> " + env_variable in client.out

        # Use relative path defined in environment variable
        env_variable = "env_variable=relative_profile"
        rel_path = os.path.join('..', 'env_rel_profile')
        assert not os.path.isabs(rel_path)
        default_profile_path = os.path.join(client.paths.profiles_path, rel_path)
        save(default_profile_path, "[settings]\nos=Windows\n[buildenv]\n" + env_variable)
        with environment_update({'CONAN_DEFAULT_PROFILE': rel_path}):
            client.run("create . --name=name --version=version --user=user --channel=channel")
            assert ">>> " + env_variable in client.out

        # Use non existing path
        profile_path = os.path.join(tmp, "this", "is", "a", "path")
        assert os.path.isabs(profile_path)
        with environment_update({'CONAN_DEFAULT_PROFILE': profile_path}):
            client.run("create . --name=name --version=version --user=user --channel=channel",
                       assert_error=True)
            assert "You need to create a default profile" in client.out

    def test_default_profile(self):
        c = TestClient()
        # Test with a profile set using and environment variable
        tmp = temp_folder()
        profile = os.path.join(tmp, 'myprofile')
        save(profile, "[settings]\nos=FreeBSD\n")
        with environment_update({'CONAN_DEFAULT_BUILD_PROFILE': profile}):
            c.run("profile show")
            assert "os=FreeBSD" in c.out  # build profiles


def test_conf_default_two_profiles():
    client = TestClient()
    save(os.path.join(client.paths.profiles_path, "mydefault"), "[settings]\nos=FreeBSD")
    save(os.path.join(client.paths.profiles_path, "mydefault_build"), "[settings]\nos=Android")
    global_conf = textwrap.dedent("""
        core:default_profile=mydefault
        core:default_build_profile=mydefault_build
        """)
    client.save_home({"global.conf": global_conf})
    client.save({"conanfile.txt": ""})
    client.run("install .")
    assert "Profile host:" in client.out
    assert "os=FreeBSD" in client.out
    assert "Profile build:" in client.out
    assert "os=Android" in client.out



import argparse
import pytest

from conan.cli.args import add_profiles_args


@pytest.mark.parametrize("argument", [
    ["options", "-o", "--options"],
    ["profile", "-pr", "--profile"],
    ["settings", "-s", "--settings"],
    ["conf", "-c", "--conf"]
])
class TestArgsParseProfile:
    """ Check argparse for profile arguments """

    @pytest.fixture(autouse=True)
    def setup(self, argument):
        self.argument = argument
        self.item, self.short_arg, self.long_arg = self.argument
        self.args_dest_build = '{}_build'.format(self.item)
        self.args_dest_host = '{}_host'.format(self.item)

    def _run_parse(self, *args):
        parser = argparse.ArgumentParser()
        add_profiles_args(parser)
        parsed_args = parser.parse_args(*args)
        build = getattr(parsed_args, self.args_dest_build)
        host = getattr(parsed_args, self.args_dest_host)
        return build, host

    def test_default(self):
        """ The old '--settings', '--profile',... refers to the build machine """
        build, host = self._run_parse([self.long_arg, "it1"])
        assert host == ["it1"]
        assert build is None

        build, host = self._run_parse([self.long_arg, "it1", self.short_arg, "it2"])
        assert host == ["it1", "it2"]
        assert build is None

    def test_build_machine(self):
        """ If provided with build suffix (':b', ':build'), those correspond to the build machine """
        long_arg = "{}:build".format(self.long_arg)
        short_arg = "{}:b".format(self.short_arg)

        build, host = self._run_parse([long_arg, "it1"])
        assert build == ["it1"]
        assert host is None

        build, host = self._run_parse([long_arg, "it1", short_arg, "it2"])
        assert build == ["it1", "it2"]
        assert host is None

    def test_mix_old_and_host_machine(self):
        """ Old arguments and new ':host' ones are composable """
        new_long_arg = "{}:host".format(self.long_arg)
        new_short_arg = "{}:h".format(self.short_arg)

        build, host = self._run_parse([new_long_arg, "it1", self.long_arg, "it2",
                                       new_short_arg, "it3", self.short_arg, "it4"])
        assert host == ["it1", "it2", "it3", "it4"]
        assert build is None

    def test_host_machine(self):
        """ If provided with host suffix (':h', ':host'), those correspond to the host machine """
        long_arg = "{}:host".format(self.long_arg)
        short_arg = "{}:h".format(self.short_arg)

        build, host = self._run_parse([long_arg, "it1"])
        assert host == ["it1"]
        assert build is None

        build, host = self._run_parse([long_arg, "it1", short_arg, "it2"])
        assert host == ["it1", "it2"]
        assert build is None

    def test_build_and_host(self):
        """ Of course, we can provide build and host in the same command line """
        build, host = self._run_parse(["{}:build".format(self.long_arg), "b1",
                                       "{}:host".format(self.long_arg), "h1"])
        assert build == ["b1"]
        assert host == ["h1"]



import os
from unittest import mock

import conan.test.utils.env


class TestToolsEnv:
    def test_environment_update_variables(self):
        with mock.patch.dict('os.environ', {}), conan.test.utils.env.environment_update({'env_var1': 'value',
                                                                        'env_var2': 'value2'}):
            assert os.environ['env_var1'] == 'value'
            assert os.environ['env_var2'] == 'value2'

    def test_environment_update_variables_without_values(self):
        with mock.patch.dict('os.environ', {'env_var1': 'value',
                                            'env_var2': 'value2'}), conan.test.utils.env.environment_update({}):
            assert os.environ['env_var1'] == 'value'
            assert os.environ['env_var2'] == 'value2'

    def test_environment_update_overwriting(self):
        with mock.patch.dict('os.environ', {'env_var1': 'value'}),\
             conan.test.utils.env.environment_update({'env_var1': 'new_value'}):
            assert os.environ['env_var1'] == 'new_value'

    def test_environment_update_unsetting_some_variables(self):
        with mock.patch.dict('os.environ', {'env_var1': 'value'}),\
             conan.test.utils.env.environment_update({'env_var1': None, 'env_var2': 'value2'}):
            assert 'env_var1' not in os.environ
            assert os.environ['env_var2'] == 'value2'

    def test_environment_update_unsetting_all_variables(self):
        with mock.patch.dict('os.environ',
                             {'env_var1': 'value',
                              'env_var2': 'value2'}),\
             conan.test.utils.env.environment_update({'env_var1': None}):
            assert 'env_var1' not in os.environ

    def test_environment_update_unsetting_non_existing_variables(self):
        with mock.patch.dict('os.environ',
                             {'env_var2': 'value2'}),\
             conan.test.utils.env.environment_update({'env_var1': None}):
            assert 'env_var1' not in os.environ



import base64
import os
from io import StringIO

from conan.api.output import ConanOutput
from conan.errors import ConanException
from conan.api.model import RecipeReference
from conan.internal.conan_app import ConanApp
from conan.internal.errors import conanfile_exception_formatter
from conan.internal.graph.graph import CONTEXT_HOST
from conan.internal.graph.profile_node_definer import initialize_conanfile_profile
from conan.internal.source import config_source
from conan.internal.util.runners import conan_run


class ReportAPI:
    """ Used to compute the differences (the "diff") between two versions or revisions, for
    both the recipe and source code.
    """
    def __init__(self, conan_api, helpers):
        self._conan_api = conan_api
        self._helpers = helpers

    def diff(self, old_reference, new_reference, remotes, old_path=None, new_path=None, cwd=None):
        """
        Compare two recipes and return the differences.

        :param old_reference: The reference of the old recipe.
        :param new_reference: The reference of the new recipe.
        :param remotes: List of remotes to search for the recipes.
        :param old_path: Optional path to the old recipe's conanfile.py.
        :param new_path: Optional path to the new recipe's conanfile.py.
        :param cwd: Current working directory, used to resolve paths.
        :return: A dictionary with the differences between the two recipes.
        """

        def _source(path_to_conanfile, reference):
            if path_to_conanfile is None:
                export_ref, cache_path = _get_ref_from_cache_or_remote(self._conan_api, reference,
                                                                       remotes)
            else:
                export_ref, cache_path = _export_recipe_from_path(self._conan_api, path_to_conanfile,
                                                                  reference, remotes, cwd)
            exported_path = self._conan_api.local.get_conanfile_path(cache_path, cwd, py=True)
            _configure_source(self._conan_api, self._helpers.hook_manager, exported_path, export_ref,
                              remotes)
            return export_ref, cache_path

        old_export_ref, old_cache_path = _source(old_path, old_reference)
        new_export_ref, new_cache_path = _source(new_path, new_reference)

        old_diff_path = os.path.abspath(os.path.join(old_cache_path, os.path.pardir)).replace("\\",
                                                                                              "/")
        new_diff_path = os.path.abspath(os.path.join(new_cache_path, os.path.pardir)).replace("\\",
                                                                                              "/")

        src_prefix = base64.b64encode(str(new_export_ref.repr_notime()).encode()).decode() + "/"
        dst_prefix = base64.b64encode(str(old_export_ref.repr_notime()).encode()).decode() + "/"

        command = (f'git diff --no-index --src-prefix "{src_prefix}" --dst-prefix "{dst_prefix}" '
                   f'"{old_diff_path}" "{new_diff_path}"')

        ConanOutput().info(
            f"Generating diff from {old_export_ref.repr_notime()} to {new_export_ref.repr_notime()} "
            f"(this might take a while)")
        ConanOutput().info(command)

        stdout, stderr = StringIO(), StringIO()
        conan_run(command, stdout=stdout, stderr=stderr)
        diff = stdout.getvalue()

        if old_path:
            self._conan_api.remove.recipe(old_export_ref)
        if new_path:
            self._conan_api.remove.recipe(new_export_ref)

        return {
            "diff": diff,
            "old_export_ref": old_export_ref,
            "new_export_ref": new_export_ref,
            "old_cache_path": old_diff_path,
            "new_cache_path": new_diff_path,
            "src_prefix": src_prefix,
            "dst_prefix": dst_prefix,
        }


def _configure_source(conan_api, hook_manager, conanfile_path, ref, remotes):
    app = ConanApp(conan_api)
    conanfile = app.loader.load_consumer(conanfile_path, name=ref.name, version=str(ref.version),
                                         user=ref.user, channel=ref.channel, graph_lock=None,
                                         remotes=remotes)
    # This profile is empty, but with the conf from global.conf
    profile = conan_api.profiles.get_profile([])
    initialize_conanfile_profile(conanfile, profile, profile, CONTEXT_HOST, False)
    # This is important, otherwise the ``conan source`` doesn't define layout and fails
    if hasattr(conanfile, "layout"):
        with conanfile_exception_formatter(conanfile, "layout"):
            conanfile.layout()

    recipe_layout = app.cache.recipe_layout(ref)
    export_source_folder = recipe_layout.export_sources()
    source_folder = recipe_layout.source()

    conanfile.folders.set_base_source(source_folder)
    conanfile.folders.set_base_export_sources(export_source_folder)
    conanfile.folders.set_base_recipe_metadata(recipe_layout.metadata())
    config_source(export_source_folder, conanfile, hook_manager)


def _get_ref_from_cache_or_remote(conan_api, reference, enabled_remotes):
    ref = RecipeReference.loads(reference)
    full_ref, matching_remote = None, False
    # The first remote is None, which means local cache for the list subapi methods
    for remote in [None] + enabled_remotes:
        if ref.revision:
            no_rrev_ref = RecipeReference.loads(reference)
            no_rrev_ref.revision = None
            try:
                remote_revisions = conan_api.list.recipe_revisions(no_rrev_ref, remote)
                if ref in remote_revisions:
                    full_ref = ref
                    matching_remote = remote
                    break
            except (Exception,):
                continue
        else:
            try:
                latest_recipe_revision = conan_api.list.latest_recipe_revision(ref, remote)
            except (Exception,):
                continue
            if full_ref is None or (latest_recipe_revision.timestamp > full_ref.timestamp):
                full_ref = latest_recipe_revision
                matching_remote = remote
    if full_ref is None or matching_remote is False:
        raise ConanException(f"No matching reference for {reference} in remotes.\n"
                             "If you want to check against a local recipe, add an "
                             "additional --{old,new}-path arg.\n")
    if matching_remote is not None:
        conan_api.download.recipe(full_ref, matching_remote)
    cache_path = conan_api.cache.export_path(full_ref)
    return full_ref, cache_path


def _export_recipe_from_path(conan_api, path_to_conanfile, reference, enabled_remotes, cwd=None):
    path = conan_api.local.get_conanfile_path(path_to_conanfile, cwd, py=True)
    ref = RecipeReference.loads(reference)
    export_ref, conanfile = conan_api.export.export(path=path,
                                                    name=ref.name, version=str(ref.version),
                                                    user=ref.user, channel=ref.channel,
                                                    lockfile=None,
                                                    remotes=enabled_remotes)
    cache_path = conan_api.cache.export_path(export_ref)
    return export_ref, cache_path



import pytest

from conan.tools.build import check_min_compiler_version
from conan.errors import ConanException, ConanInvalidConfiguration
from conan.test.utils.mocks import MockSettings, ConanFileMock


@pytest.mark.parametrize("compiler,compiler_version,restrictions,should_raise", [
    ("clang", "14", (("clang", "14", "coroutines"), ("gcc", "13", "alignas")), False),
    ("gcc", "13", (("clang", "14", "coroutines"), ("gcc", "13", "alignas")), False),
    ("gcc", "14", (("clang", "14", "coroutines"), ("gcc", "13", "alignas")), False),
    ("msvc", "192", (("msvc", "192", "reason"), ("clang", "14", "coroutines")), False),

    ("clang", "13", (("clang", "14", "coroutines"), ("gcc", "13", "alignas")), True),
    ("gcc", "12", (("clang", "14", "coroutines"), ("gcc", "13", "alignas")), True),
    ("msvc", "191", (("msvc", "192", "reason"), ("clang", "14", "coroutines")), True),

    ("emcc", "1.0", (("gcc", "13", "alignas"),), False),
    (None, "12", (("gcc", "13", "alignas"),), True),
    ("gcc", None, (("gcc", "13", "alignas"),), True),
])
def test_check_min_compiler_version(compiler, compiler_version, restrictions, should_raise):
    settings = MockSettings({"compiler": compiler, "compiler.version": compiler_version})
    conanfile = ConanFileMock(settings)

    if should_raise:
        if compiler is None or compiler_version is None:
            with pytest.raises(ConanException):
                check_min_compiler_version(conanfile, restrictions)
        else:
            with pytest.raises(ConanInvalidConfiguration):
                check_min_compiler_version(conanfile, restrictions)
    else:
        check_min_compiler_version(conanfile, restrictions)
